@@ -1,5 +1,5 @@
 (* $I1: Unison file synchronizer: src/osx.mli $ *)
-(* $I2: Last modified by vouillon on Tue, 31 Aug 2004 11:33:38 -0400 $ *)
+(* $I2: Last modified by vouillon on Thu, 25 Nov 2004 16:01:48 -0500 $ *)
 (* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
 
 val init : bool -> unit
@@ -15,9 +15,9 @@ type 'a ressInfo
 type ressStamp = unit ressInfo
 type info =
   { ressInfo : (string * int64) ressInfo;
-    typeCreator : string }
+    finfo : string }
 
-val getFileInfos : Fspath.t -> Path.local -> [> `FILE ] -> info
+val getFileInfos : Fspath.t -> Path.local -> [> `DIRECTORY | `FILE ] -> info
 val setFileInfos : Fspath.t -> Path.local -> string -> unit
 
 val ressUnchanged :
@@ -33,5 +33,5 @@ val stamp : info -> ressStamp
 
 val appleDoubleFile : Fspath.t -> Path.local -> string
 
-val openRessIn : Fspath.t -> Path.local -> Unix.file_descr
-val openRessOut : Fspath.t -> Path.local -> Uutil.Filesize.t -> Unix.file_descr
+val openRessIn : Fspath.t -> Path.local -> in_channel
+val openRessOut : Fspath.t -> Path.local -> Uutil.Filesize.t -> out_channel
