@@ -1,5 +1,5 @@
 (* $I1: Unison file synchronizer: src/props.ml $ *)
-(* $I2: Last modified by bcpierce on Mon, 06 Sep 2004 14:48:05 -0400 $ *)
+(* $I2: Last modified by bcpierce on Sat, 27 Nov 2004 09:22:40 -0500 $ *)
 (* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
 
 let debug = Util.debug "props"
@@ -494,11 +494,16 @@ let strip t = t
 
 let diff t t' = if similar t t' then None else t'
 
+let zeroes = "\000\000\000\000\000\000\000\000"
+
 let toString t =
   match t with
-    None | Some "" -> ""
-  | Some s         -> " " ^ String.escaped (String.sub s 0 4) ^
-                      " " ^ String.escaped (String.sub s 4 4)
+    Some s when s.[0] = 'F' && String.sub (s ^ zeroes) 1 8 <> zeroes ->
+      let s = s ^ zeroes in
+      " " ^ String.escaped (String.sub s 1 4) ^
+      " " ^ String.escaped (String.sub s 5 4)
+  | _ ->
+      ""
 
 let syncedPartsToString = toString
 
@@ -508,8 +513,12 @@ let set fspath path kind t =
   | Some t -> Osx.setFileInfos fspath path t
 
 let get stats info =
-  if Prefs.read Osx.rsrc && stats.Unix.LargeFile.st_kind = Unix.S_REG then
-    Some info.Osx.typeCreator
+  if
+    Prefs.read Osx.rsrc &&
+    (stats.Unix.LargeFile.st_kind = Unix.S_REG ||
+     stats.Unix.LargeFile.st_kind = Unix.S_DIR)
+  then
+    Some info.Osx.finfo
   else
     None
 
@@ -615,9 +624,9 @@ let get stats infos =
 let set fspath path kind p =
   Uid.set fspath path kind p.uid;
   Gid.set fspath path kind p.gid;
+  TypeCreator.set fspath path kind p.typeCreator;
   Perm.set fspath path kind p.perm;
-  Time.set fspath path kind p.time;
-  TypeCreator.set fspath path kind p.typeCreator
+  Time.set fspath path kind p.time
 
 let init someHostIsRunningWindows =
   Perm.init someHostIsRunningWindows;
