@@ -1,6 +1,6 @@
 (* $I1: Unison file synchronizer: src/props.mli $ *)
-(* $I2: Last modified by bcpierce on Sun, 24 Mar 2002 11:24:03 -0500 $ *)
-(* $I3: Copyright 1999-2002 (see COPYING for details) $ *)
+(* $I2: Last modified by vouillon on Mon, 14 Jun 2004 16:38:56 -0400 $ *)
+(* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
 
 (* File properties: time, permission, length, etc. *)
 
@@ -13,13 +13,13 @@ val strip : t -> t
 val diff : t -> t -> t
 val toString : t -> string
 val syncedPartsToString : t -> string
-val set : Fspath.t -> Path.t -> [`Set | `Update] -> t -> unit
-val get : Unix.stats -> t
+val set : Fspath.t -> Path.local -> [`Set | `Update] -> t -> unit
+val get : Unix.LargeFile.stats -> Osx.info -> t
 val init : bool -> unit
 
 val same_time : t -> t -> bool
-val length : t -> Uutil.filesize
-val setLength : t -> Uutil.filesize -> t
+val length : t -> Uutil.Filesize.t
+val setLength : t -> Uutil.Filesize.t -> t
 val time : t -> float
 val setTime : t -> float -> t
 val perms : t -> int
