@@ -1,5 +1,5 @@
 (* $I1: Unison file synchronizer: src/uicommon.ml $ *)
-(* $I2: Last modified by bcpierce on Sun, 22 Aug 2004 22:29:04 -0400 $ *)
+(* $I2: Last modified by bcpierce on Sat, 27 Nov 2004 09:22:40 -0500 $ *)
 (* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
 
 open Common
@@ -102,6 +102,13 @@ let repeat =
      ^ "modified, Unison will wait, checking them again every few seconds."
     )
 
+let retry =
+  Prefs.createInt "retry" 0
+    "re-try failed synchronizations N times (text interface only)"
+    ("Setting this preference causes the text-mode interface to try again to synchronize "
+     ^ "updated paths where synchronization fails.  Each such path will be tried N times."
+    )
+
 (**********************************************************************
                          Formatting functions
  **********************************************************************)
@@ -341,6 +348,8 @@ let checkCaseSensitivity () =
   Globals.allRootsMap (fun r -> architecture r ()) >>= (fun archs ->
   let someHostIsRunningWindows =
     Safelist.exists (fun (isWin, _) -> isWin) archs in
+  let allHostsAreRunningWindows =
+    Safelist.for_all (fun (isWin, _) -> isWin) archs in
   let someHostRunningOsX =
     Safelist.exists (fun (_, isOSX) -> isOSX) archs in
   let someHostIsCaseInsensitive =
@@ -348,6 +357,8 @@ let checkCaseSensitivity () =
   Case.init someHostIsCaseInsensitive;
   Props.init someHostIsRunningWindows;
   Osx.init someHostRunningOsX;
+  Prefs.set Update.someHostIsRunningWindows someHostIsRunningWindows;
+  Prefs.set Update.allHostsAreRunningWindows allHostsAreRunningWindows;
   return ())
 
 (* ---- *)
