@@ -1,5 +1,5 @@
 (* $I1: Unison file synchronizer: src/uicommon.mli $ *)
-(* $I2: Last modified by bcpierce on Sun, 22 Aug 2004 22:29:04 -0400 $ *)
+(* $I2: Last modified by bcpierce on Sat, 27 Nov 2004 09:22:40 -0500 $ *)
 (* $I3: Copyright 1999-2004 (see COPYING for details) $ *)
 
 (* Kinds of UI *)
@@ -35,6 +35,9 @@ val profileLabel : string Prefs.t
 (* User preference: Synchronize repeatedly *)
 val repeat : string Prefs.t
 
+(* User preference: Try failing paths N times *)
+val retry : int Prefs.t
+
 (* Format the information about current contents of a path in one replica *)
 val details2string : Common.reconItem -> string -> string
 
